/*************************************************************************
 * This file is part of Crosroads 1.38, a load balancer and fail over
 * utility for TCP. Copyright (c) Karel Kubat, distributed under GPL.
 * Visit http://crossroads.e-tunity.com for information.
 *************************************************************************/
#include "crossroads.h"

int ipf_match (IpFilter f) {
    static int client_ip_nr;
    int counter, val;
    char *tmp, *cp;
    
    /* Build up the actual client IP as an int.
     * We need to do this in reverse order; e.g. 1.2.3.4 needs to become
     * 4<<24 | 3<<16 | 2<<8 | 1. Reason is that when mask /16 applies,
     * we can OR this value with 16, and we'll keep 2<<8 | 1, the only
     * significant digits. */
    if (!client_ip_nr) {
	tmp = xstrdup (client_ip);
	counter = 0;
	for (counter = 0, cp = strtok (tmp, ".");
	     cp;
	     counter += 8, cp = strtok (0, ".")) {
	    if (sscanf (cp, "%d", &val) < 1) {
		free (tmp);
		return (1);
	    }
	    val <<= counter;
	    client_ip_nr |= val;
	    /* msg ("ipf_match: val 0x%x, client_ip_nr 0x%x",
	     * 	     val, client_ip_nr); */
	}
    }
    free (tmp);

    /* Here's the comparison. */
    /* msg ("ipf_match: filter ip/mask 0x%x/0x%x, "
     *      "filter res 0x%x, client res 0x%x",
     *      f.ip, f.mask, 
     *      (f.ip & f.mask), (client_ip_nr & f.mask));
     */
    return ( (f.ip & f.mask) == (client_ip_nr & f.mask) );
}
