/*************************************************************************
 * This file is part of Crosroads 1.38, a load balancer and fail over
 * utility for TCP. Copyright (c) Karel Kubat, distributed under GPL.
 * Visit http://crossroads.e-tunity.com for information.
 *************************************************************************/

#define EXTERN
#include "crossroads.h"

/* Explicit initializations. */

int log_facility = LOG_DAEMON;

StateStringMap statestringmap[] = {
    { st_available, "available" },
    { st_available, "up" },
    { st_unavailable, "UNAVAILABLE" },
    { st_down, "DOWN" },
    { st_waking, "WAKING" },
    { st_intermediate, "INTERMEDIATE" },
    { st_unknown, "UNKNOWN" },
    { st_unknown, 0 }
};

int relevant_sigs[] = {			/* signals relevant to this app */
    SIGHUP,				/* either caught or ignored, */
    SIGINT,				/* depending on the stage */
    SIGQUIT,
    SIGABRT,
    SIGKILL,
    SIGPIPE,
    SIGTERM,
    0,
};

typedef struct {
    char *arg;				/* cmdline argument */
    int narg;				/* required remaining args */
    int needconf;			/* configuration file required? */
    int (*handler)(int ac, char **av);	/* action handler */
} Handler;

int main (int argc, char **argv) {
    int opt;
    unsigned i;    
    static Handler handler[] = {
	/*  Argument	     Nr.args  Needconf  Handler function  */
	/*		   -1=no check				  */
	{ "services",		0,	1,	show_services	},
	{ "status",	       -1,	1,	show_status	},
	{ "stop",		0,	1,	stop_daemon	},
	{ "start", 		0,	1,	serve		},
	{ "restart",		0,	1,	restart 	},
	{ "tell", 		3,	1,	tell_service 	},
	{ "configtest",		0,	1,	configtest 	},
    };

    /* Remember original ac/av/ep */
    org_argc = argc;
    org_argv = argv;

    /* Parse options. */
    config_file = DEFAULT_CONF;
    while ( (opt = getopt (argc, argv, "?c:fhvi:Val:stC")) > 0 )
	switch (opt) {
	case 'a':
	    log_activity++;
	    break;
	case 'c':
	    config_file = optarg;
	    break;
	case 'v':
	    flag_verbose++;
	    break;
	case 'i':
	    iflag_present++;
	    break;
	case 'l':
	    switch (atoi (optarg)) {
	    case 0:
		log_facility = LOG_LOCAL0;
		break;
	    case 1:
		log_facility = LOG_LOCAL1;
		break;
	    case 2:
		log_facility = LOG_LOCAL2;
		break;
	    case 3:
		log_facility = LOG_LOCAL3;
		break;
	    case 4:
		log_facility = LOG_LOCAL4;
		break;
	    case 5:
		log_facility = LOG_LOCAL5;
		break;
	    case 6:
		log_facility = LOG_LOCAL6;
		break;
	    case 7:
		log_facility = LOG_LOCAL7;
		break;
	    default:
		usage();
	    }
	    break;
	case 's':
	    sloppyportbind++;
	    break;
	case 't':
	    tabular_status++;
	    break;
	case 'C':
	    show_config();
	    exit (0);
	case 'V':
	    puts (VER);
	    exit (0);
	case '?':
	case 'h':
	default:
	    usage ();
	}

    /* We need at last one argument: the action */
    if (optind >= argc)
	usage();

    /* Act on the argument. */
    for (i = 0; i < sizeof(handler) / sizeof(Handler); i++) {
	/* Match the handler action string with argv, and match the
	 * required remaining arguments. If they match..
	 * scan the configuration (if needed) and GO!
	 */
	if (!strcmp (argv[optind], handler[i].arg) &&
	    (handler[i].narg == -1 ||
	     handler[i].narg == argc - optind - 1)) {
	    if (handler[i].needconf) {
		msg ("Parsing configuration %s", config_file);
		if (! (yyin = fopen (config_file, "r")) )
		    error ("Cannot read configuration %s: %s",
			   config_file, strerror(errno));
		yyparse();
		fclose (yyin);
	    }
	    if (!handler[i].handler (argc - optind, argv + optind + 1))
		error ("'%s' failed", argv[optind]);
	    return (0);
	}
    }

    usage();
    return (1);
}
