/*************************************************************************
 * This file is part of Crosroads 1.38, a load balancer and fail over
 * utility for TCP. Copyright (c) Karel Kubat, distributed under GPL.
 * Visit http://crossroads.e-tunity.com for information.
 *************************************************************************/
#include "crossroads.h"

static char *timestr (double nsec) {
    static char *buf;
    char *tmp;
    
    free (buf);
    buf = 0;

    if (nsec > 3600) {
	buf = str_printf ("%uh", (unsigned) (nsec / 3600));
	nsec = fmod (nsec, 3600);
    }
    if (nsec > 60) {
	tmp = str_printf ("%um", (unsigned) (nsec / 60));
	nsec = fmod (nsec, 60);
	buf = xstrcat (buf, tmp);
	free (tmp);
    }
    tmp = str_printf ("%.2fs", nsec);
    buf = xstrcat (buf, tmp);
    free (tmp);

    return (buf);
}

static char *bytestr (double nbytes) {
    static char *buf;

    free (buf);
    
    if (nbytes > (double)1024*1024*1024*1024)
	buf = str_printf ("%.2fTb",
			  nbytes / ((double) 1024*1024*1024*1024));
    else if (nbytes > 1024*1024*1024)
	buf = str_printf ("%.2fGb",
			  nbytes / ((double) 1024*1024*1024));
    else if (nbytes > 1024*1024)
	buf = str_printf ("%.2fMb",
			  nbytes / ((double) 1024*1024));
    else if (nbytes > 1024)
	buf = str_printf ("%.2fKb",
			  nbytes / (double) 1024);
    else
	buf = str_printf ("%gb", nbytes);

    return (buf);
}

int show_status (int ac, char **av) {
    int i, j, services_shown = 0;

    for (i = 0; i < nservice; i++) {
	if (ac > 1 && strcmp (service[i].name, av[0]))
	    continue;
	if (services_shown++)
	    putchar ('\n');
	alloc_reporter (service + i, 0);
	msg ("Reporting service %s (pid = %d)",
	     service[i].name, servicereport->pid);
	if (tabular_status)
	    printf ("%s ", service[i].name);
	else
	    printf ("Service      : %s, %d live connections, "
		    "last backend %d\n",
		    service[i].name, servicereport->nclients,
		    servicereport->last_backend);
	for (j = 0; j < service[i].nbackend; j++) {
	    if (ac > 2 && strcmp (service[i].backend[j].name, av[1]))
		continue;
	    if (tabular_status)
		printf ("%s=%s ", service[i].backend[j].name,
			state_to_string(servicereport->backendstate[j].avail));
	    else {
		printf ("  Backend %2d : %s is %s, %u live connections\n"
			"    Stats    : %lu failures out of %lu connections",
			j,
			service[i].backend[j].name,
			state_to_string (servicereport->backendstate[j].avail),
			servicereport->backendstate[j].nclients,
			servicereport->backendstate[j].failures,
			servicereport->backendstate[j].totuses);
		if (service[i].type == type_http)
		    printf (", %lu sessions,",
			    servicereport->backendstate[j].sessions);
		printf ("\n"
			"               usage %s, %s",
			timestr (servicereport->backendstate[j].nsec),
			bytestr (servicereport->backendstate[j].nbytes));
		if (service[i].dispatchover)
		    printf (", avg %s, %s",
			  timestr(servicereport->backendstate[j].avg_nsec),
			  bytestr(servicereport->backendstate[j].avg_nbytes));
		putchar ('\n');
	    }
	}
	
	if (shmdt (servicereport) < 0)
	    error ("Failure releasing reporter memory: %s",
		   strerror(errno));
    }
    if (tabular_status)
	putchar ('\n');

    return (1);
}
